// addrspace.h 
//	Data structures to keep track of executing user programs 
//	(address spaces).
//
//	For now, we don't keep any information about address spaces.
//	The user level CPU state is saved and restored in the thread
//	executing the user program (see thread.h).
//
// Copyright (c) 1992-1993 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

#ifndef ADDRSPACE_H
#define ADDRSPACE_H

#include "copyright.h"
#include "filesys.h"
#include "list.h"

#define UserStackSize		1024 	// increase this as necessary!

class Thread;

class FD
{
public:
	OpenFile *openfile;
	int fd;
	FD *next;
};

class FDList
{
private:
	FD *first;
	FD *last;
public:
	FDList()
	{
		first=NULL;
		last=NULL;
	}

	~FDList()
	{
		FD *rmfd;
		while(first!=NULL)
		{
			rmfd=first;
			first=first->next;
			delete rmfd;
		}
		first=NULL;
		last=NULL;
	}

	int addFD(FD *fdfile)
	{
		if(fdfile==NULL)
			return -1;
		if(last==NULL)
		{
			first=fdfile;
			last=fdfile;
			first->next=NULL;
		}
		else
		{
			last->next=fdfile;
			last=last->next;
			last->next=NULL;
		}
		return fdfile->fd;
	}

	FD *getFD(int fd)
	{
		FD *i;
		for(i=first;i!=NULL;i=i->next)
			if(i->fd==fd)
				return i;
		return NULL;
	}

	void printAll()
	{
		FD *i;
		printf("Dump\n");
		for(i=first;i!=NULL;i=i->next)
			printf("%d : %p\n",i->fd,i->openfile);
	}

	void removeFD(int fd)
	{
		FD *tmp;
		if(first==NULL)
			return;
		if(first==last)
		{
			if(first->fd==fd)
			{
				first=NULL;
				last=NULL;
			}
		}
		else if(first->fd==fd)
		{
			tmp=first;
			first=first->next;
			delete tmp;
		}
		else
		{
			FD *i;
			FD *prev;
			for(i=first;i!=NULL;prev=i,i=i->next)
				if(i->fd==fd)
				{
					tmp=i;
					prev->next=i->next;
					if(i==last)
						last=prev;
					delete i;
				}
		}
	}
};

class AddrSpace {
  public:
    AddrSpace(OpenFile *executable);	// Create an address space,
					// initializing it with the program
					// stored in the file "executable"
    ~AddrSpace();			// De-allocate an address space

    void InitRegisters();		// Initialize user-level CPU registers,
					// before jumping to user code

    void SaveState();			// Save/restore address space-specific
    void RestoreState();		// info on a context switch 
    int getPhysicalAddr(int virtual_addr);
    int getVirtualAddr(int physical_addr);
    int addFile(OpenFile *openfile);
    OpenFile *getFile(int fd);
    void removeFile(int fd);
    int user2kernel(int addr,char *string,int length);
    int kernel2user(int addr,char *string,int length);
    int user2kernel(int addr,char *string);
    int kernel2user(int addr,char *string);
    void join(Thread *t);
    Thread *removeJoin();


    TranslationEntry *pageTable;	// Assume linear page table translation
					// for now!
    unsigned int numPages;		// Number of pages in the virtual 
					// address space
    FDList fdlist;
    List *joinList;
    int maxfd;
    int argc;
    char **argv;
};

#endif // ADDRSPACE_H
